// Compile with: gcc mkidx.cc -lstdcpp -lgpp -s

#include <strng.h>
#include <iostream.h>
#include <stl.h>

typedef list<String> stringList;
typedef list<String>::const_iterator stringListIterator;

void main(int argc, char *argv[] )
{
   bool opt_c = false;
   bool opt_l = false;

   String currLine;
   String attrStr;
   String typeStr;
   String locAttrs      = "";
   String locMarkup     = "";
   String xrefAttrNames = "";
   String xrefAttrs     = "";
   String xrefMarkup    = "";

   stringList staList;

   // process command line options
   for( int i = 1; i < argc; i++ ) {
      String currSwitch = argv[i];
      if( currSwitch == "-c" ) {
         opt_c = true;
      } else if ( currSwitch == "-l" ) {
         opt_l = true;
      } else {
         cerr << "unsupported command line switch " << argv[i] << endl;
      }
   }

   // read complete STA file
   while( readline( cin, currLine ) ) staList.push_back( currLine );
   
   // sort read STA file and eliminate duplicate lines
   staList.sort();
   staList.unique();

   // find all attributes
   for( stringListIterator i = staList.begin(); i != staList.end(); ++i ) {
      String e = *i;
      attrStr = e.before( '\t' );
      typeStr = e.after( '\t' );

      if( typeStr == "locref" ) {
         if( attrStr != "" ) {   
	    locAttrs  += "\"" + attrStr + "\" ";
	    locMarkup += "(markup-locref :open \"\\" + attrStr +
		         "{\" :close \"}\" :attr \"" + attrStr + "\")\n";
         }
      } else {
	if ( attrStr != "see" ) {
	    xrefAttrNames += "\"" + attrStr + "\" ";
	    xrefAttrs     += "(define-crossref-class \"" + attrStr + "\")\n";
	    xrefMarkup    += "(markup-crossref-list :open \"\\" + attrStr +
		             "{\" :close \"}{}\")\n";
        }
      }

   }

   // write XDY file
   cout << ";; This file is generated automatically." << endl;
   cout << ";;" << endl;
   cout << ";; It is a temporary style-file that is necessary to run `xindy' properly." << endl;

   if (opt_c) { 

      cout << ";; These lines implement the blank compression defined by" << endl;
      cout << ";; the -c switch of makeindex." << endl;
      cout << "" << endl;
      cout << "(merge-rule \" +\"  \" \" :eregexp)" << endl;
      cout << "(merge-rule \"^[ \t]+\" \"\"   :again :eregexp)" << endl;
      cout << "(merge-rule \"[ \t]+$\" \"\" :again :eregexp)" << endl << endl;
      
   }


   if (opt_l) {
      cout << "(sort-rule \" \" \"\")" << endl << endl;
   }


      cout << ";; The following attributes were detected." << endl;
      cout << "" << endl;
      cout << "(define-attributes ((" + locAttrs + "\"default\")))" << endl;
      cout << "" << endl;
      cout << ";; This is the markup as generated by makeindex." << endl;
      cout << "" << endl;
      cout << locMarkup << endl;
      cout << "" << endl;
      cout << ";; The following cross-referernces were detected." << endl;
      cout << "" << endl;
      cout << "(define-crossref-class \"see\")" << endl;
      cout << "" << endl;
      cout << xrefAttrs << endl;
      cout << "" << endl;
      cout << ";; This is the markup as generated by makeindex." << endl;
      cout << "" << endl;
      cout << "(markup-crossref-list :open \"see{\" :close \"}{}\" :class \"see\")" << endl;
      cout << "" << endl;
      cout << xrefMarkup << endl;
      cout << "" << endl;
      cout << ";; Here are the location-classes" << endl;
      cout << "" << endl;
      cout << "(define-location-class \"arabic-page-numbers\" (\"arabic-numbers\"))" << endl;
      cout << "(define-location-class \"roman-page-numbers\"  (\"roman-numbers-lowercase\"))" << endl;
      cout << "(define-location-class \"Roman-page-numbers\"  (\"roman-numbers-uppercase\"))" << endl;
      cout << "(define-location-class \"alpha-page-numbers\"  (\"alpha\"))" << endl;
      cout << "(define-location-class \"Alpha-page-numbers\"  (\"ALPHA\"))" << endl;
      cout << "" << endl;
      cout << "(define-location-class-order (\"roman-page-numbers\"" << endl;
      cout << "			      \"arabic-page-numbers\"" << endl;
      cout << "			      \"alpha-page-numbers\"" << endl;
      cout << "			      \"Roman-page-numbers\"" << endl;
      cout << "			      \"Alpha-page-numbers\"" << endl;
      cout << "			      \"see\" " + xrefAttrNames + "))" << endl;
      cout << "" << endl;
      cout << ";; Now load the rest of the makeindex-module..." << endl;
      cout << "" << endl;
      cout << "(require \"tex/makeidx4.xdy\")" << endl;
      cout << "" << endl;
      cout << ";; End" << endl;

}
